@@ -1,21 +1,8 @@
-// File_Aac - Info for AAC files
-// Copyright (C) 2007-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 // Pre-compilation
@@ -35,6 +22,9 @@
 
 //---------------------------------------------------------------------------
 #include "MediaInfo/Audio/File_Aac.h"
+#if MEDIAINFO_ADVANCED
+    #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
+#endif //MEDIAINFO_ADVANCED
 #include <cmath>
 using namespace std;
 //---------------------------------------------------------------------------
@@ -47,19 +37,19 @@ namespace MediaInfoLib
 //***************************************************************************
 
 //---------------------------------------------------------------------------
-extern const int32u Aac_sampling_frequency[]=
+extern const int32u Aac_sampling_frequency[13]=
 {96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050,
- 16000, 12000, 11025,  8000,  7350,     0,     0,     0,};
+ 16000, 12000, 11025,  8000,  7350};
 
 //---------------------------------------------------------------------------
-const char* Aac_Adts_ID[]=
+static const char* Aac_Adts_ID[]=
 {
     "MPEG-4",
     "MPEG-2",
 };
 
 //---------------------------------------------------------------------------
-const char* Aac_Format(int8u ID)
+static const char* Aac_Format(int8u ID)
 {
     switch (ID)
     {
@@ -166,7 +156,7 @@ const char* Aac_audioObjectType(int8u audioObjectType)
 }
 
 //---------------------------------------------------------------------------
-const int8u Aac_Channels[]=
+static const int8u Aac_Channels[]=
 {
     0,
     1,
@@ -187,7 +177,7 @@ const int8u Aac_Channels[]=
 };
 
 //---------------------------------------------------------------------------
-const char* Aac_ChannelConfiguration[]=
+static const char* Aac_ChannelConfiguration[]=
 {
     "",
     "Front: C",
@@ -208,7 +198,7 @@ const char* Aac_ChannelConfiguration[]=
 };
 
 //---------------------------------------------------------------------------
-const char* Aac_ChannelConfiguration2[]=
+static const char* Aac_ChannelConfiguration2[]=
 {
     "",
     "1/0/0",
@@ -228,7 +218,20 @@ const char* Aac_ChannelConfiguration2[]=
     "",
 };
 
-int8u Aac_AudioSpecificConfig_sampling_frequency_index(const int32u sampling_frequency)
+//---------------------------------------------------------------------------
+static const char* Aac_ChannelLayout[]=
+{
+    "",
+    "C",
+    "L R",
+    "C L R",
+    "C L R Cs",
+    "C L R Ls Rs",
+    "C L R Ls Rs LFE",
+    "C L R Ls Rs Lrs Rrs LFE",
+};
+
+int8u Aac_AudioSpecificConfig_sampling_frequency_index(const int64s sampling_frequency)
 {
     if (sampling_frequency>=92017) return 0;
     if (sampling_frequency>=75132) return 1;
@@ -251,16 +254,19 @@ void File_Aac::AudioSpecificConfig (size_t End)
     bool    sbrData=false, sbrPresentFlag=false, psData=false, psPresentFlag=false;
     Element_Begin1("AudioSpecificConfig");
     GetAudioObjectType(audioObjectType,                         "audioObjectType");
+    Infos["CodecID"].From_Number(audioObjectType);
     Get_S1 (4, sampling_frequency_index,                        "samplingFrequencyIndex"); Param_Info1(Aac_sampling_frequency[sampling_frequency_index]);
     if (sampling_frequency_index==0xF)
     {
-        Get_S3 (24, sampling_frequency,                         "samplingFrequency");
-        sampling_frequency_index=Aac_AudioSpecificConfig_sampling_frequency_index(sampling_frequency);
+        int32u samplingFrequency;
+        Get_S3 (24, samplingFrequency,                          "samplingFrequency");
+        Frequency_b=samplingFrequency;
+        sampling_frequency_index=Aac_AudioSpecificConfig_sampling_frequency_index(Frequency_b);
     }
     else
-        sampling_frequency=Aac_sampling_frequency[sampling_frequency_index];
+        Frequency_b=Aac_sampling_frequency[sampling_frequency_index];
     Get_S1 (4, channelConfiguration,                            "channelConfiguration"); Param_Info1(Aac_ChannelConfiguration[channelConfiguration]);
-    if (audioObjectType==0x05 || audioObjectType==0x29)
+    if (audioObjectType==5 || audioObjectType==29)
     {
         extensionAudioObjectType=5;
         sbrPresentFlag=true;
@@ -354,13 +360,15 @@ void File_Aac::AudioSpecificConfig (size_t End)
             //~ break;
         default:
             Element_Begin1("not implemented part");
-            Skip_BS(Data_BS_Remain()-End,                       "(Not implemented)");
+            Skip_BS(Data_BS_Remain()-((End==(size_t)-1)?0:End), "(Not implemented)");
             Element_End0();
-            if (Mode==File_Aac::Mode_ADIF || Mode==File_Aac::Mode_ADTS)
-                File__Tags_Helper::Finish();
-            else
-                File__Analyze::Finish();
-            Frame_Count=(size_t)-1; //Forcing not to parse following data anymore (if ParseSpeed==1)
+            FILLING_BEGIN()
+                if (Mode==File_Aac::Mode_ADIF || Mode==File_Aac::Mode_ADTS)
+                    File__Tags_Helper::Finish();
+                else if (Mode==Mode_AudioSpecificConfig)
+                    File__Analyze::Finish();
+                Frame_Count=(size_t)-1; //Forcing not to parse following data anymore (if ParseSpeed==1)
+            FILLING_END()
             return;
     }
 
@@ -388,12 +396,16 @@ void File_Aac::AudioSpecificConfig (size_t End)
                 if ( ! directMapping )
                 {
                     Element_Begin1("not implemented part");
-                    Skip_BS(Data_BS_Remain()-End,               "(Not implemented)");
+                    Skip_BS(Data_BS_Remain()-((End==(size_t)-1)?0:End), "(Not implemented)");
                     Element_End0();
                     if (Mode==File_Aac::Mode_ADIF || Mode==File_Aac::Mode_ADTS)
                         File__Tags_Helper::Finish();
                     else
+                    {
+                        if (Mode==Mode_LATM)
+                            File__Analyze::Accept();
                         File__Analyze::Finish();
+                    }
                     Frame_Count=(size_t)-1; //Forcing not to parse following data anymore (if ParseSpeed==1)
                     return;
                 }
@@ -434,7 +446,7 @@ void File_Aac::AudioSpecificConfig (size_t End)
                     }
                 }
             }
-            if ( extensionAudioObjectType == 22 )
+            if ( extensionAudioObjectType == 29 )
             {
                 Get_SB(sbrPresentFlag,                          "sbrPresentFlag");
                 if (sbrPresentFlag)
@@ -461,45 +473,40 @@ void File_Aac::AudioSpecificConfig (size_t End)
         Skip_BS(Data_BS_Remain()-End,                           LastByte?"Unknown":"Padding");
     }
 
-    FILLING_BEGIN()
-        AudioSpecificConfig_OutOfBand (sampling_frequency, audioObjectType, sbrData, psData, sbrPresentFlag, psPresentFlag);
-
-        //Parsing the rest
-        if (audioObjectType!=2) //We continue only if AAC LC (in order to detect SBR and PS)
-        {
-            File__Analyze::Finish();
-            Frame_Count=(size_t)-1; //Forcing not to parse following data anymore (if ParseSpeed==1)
-        }
+    FILLING_BEGIN();
+        AudioSpecificConfig_OutOfBand (Frequency_b, audioObjectType, sbrData, psData, sbrPresentFlag, psPresentFlag);
     FILLING_END();
 }
 
 //---------------------------------------------------------------------------
-void File_Aac::AudioSpecificConfig_OutOfBand (int32u sampling_frequency_, int8u audioObjectType_, bool sbrData, bool psData, bool sbrPresentFlag, bool psPresentFlag)
+void File_Aac::AudioSpecificConfig_OutOfBand (int64s sampling_frequency_, int8u audioObjectType_, bool sbrData, bool psData, bool sbrPresentFlag, bool psPresentFlag)
 {
-    if (sampling_frequency==(int32u)-1 && sampling_frequency_) //Only if not yet set
+    if (!Frequency_b && sampling_frequency_) //Only if not yet set
     {
-        sampling_frequency=sampling_frequency_;
-        sampling_frequency_index=Aac_AudioSpecificConfig_sampling_frequency_index(sampling_frequency);
+        Frequency_b=sampling_frequency_;
+        sampling_frequency_index=Aac_AudioSpecificConfig_sampling_frequency_index((int32u)Frequency_b);
     }
 
     if (audioObjectType_==(int8u)-1)
     {
         if (audioObjectType==(int8u)-1)
             return; //All data is not yet available
-        if (Infos.find("Format_Settings_SBR")!=Infos.end())
+        std::map<std::string, Ztring>::const_iterator i=Infos.find("Format_Settings_SBR");
+        if (i!=Infos.end())
         {
             sbrData=true;
-            sbrPresentFlag=Infos["Format_Settings_SBR"].find(__T("Yes"))!=string::npos;
+            sbrPresentFlag=i->second.find(__T("Yes"))!=string::npos;
         }
         else
         {
             sbrData=false;
             sbrPresentFlag=false;
         }
-        if (Infos.find("Format_Settings_PS")!=Infos.end())
+        i = Infos.find("Format_Settings_PS");
+        if (i!=Infos.end())
         {
             psData=true;
-            psPresentFlag=Infos["Format_Settings_PS"].find(__T("Yes"))!=string::npos;
+            psPresentFlag=i->second.find(__T("Yes"))!=string::npos;
         }
         else
         {
@@ -510,7 +517,8 @@ void File_Aac::AudioSpecificConfig_OutOfBand (int32u sampling_frequency_, int8u
     else
         audioObjectType=audioObjectType_;
 
-    Infos["SamplingRate"].From_Number(sampling_frequency);
+    if (Frequency_b)
+        Infos["SamplingRate"].From_Number(Frequency_b);
     Infos["Format"].From_Local(Aac_Format(audioObjectType));
     Infos["Format_Profile"].From_Local(Aac_Format_Profile(audioObjectType));
     Infos["Codec"].From_Local(Aac_audioObjectType(audioObjectType));
@@ -519,30 +527,36 @@ void File_Aac::AudioSpecificConfig_OutOfBand (int32u sampling_frequency_, int8u
         Infos["Channel(s)"].From_Number(Aac_Channels[channelConfiguration]);
         Infos["ChannelPositions"].From_Local(Aac_ChannelConfiguration[channelConfiguration]);
         Infos["ChannelPositions/String2"].From_Local(Aac_ChannelConfiguration2[channelConfiguration]);
+        Infos["ChannelLayout"].From_Local(Aac_ChannelLayout[channelConfiguration]);
     }
 
-    if (sbrPresentFlag)
+    if (sbrPresentFlag || !Infos["Format_Settings_SBR"].empty())
     {
         Infos["Format_Profile"]=__T("HE-AAC");
-        Ztring SamplingRate=Infos["SamplingRate"];
-        Infos["SamplingRate"].From_Number((extension_sampling_frequency_index==(int8u)-1)?(sampling_frequency*2):extension_sampling_frequency, 10);
-        if (MediaInfoLib::Config.LegacyStreamDisplay_Get())
+        Ztring SamplingRate_Previous=Infos["SamplingRate"];
+        int32u SamplingRate=(extension_sampling_frequency_index==(int8u)-1)?(((int32u)Frequency_b)*2):extension_sampling_frequency;
+        if (SamplingRate)
         {
-            Infos["Format_Profile"]+=__T(" / LC");
-            Infos["SamplingRate"]+=__T(" / ")+SamplingRate;
+            Infos["SamplingRate"].From_Number(SamplingRate, 10);
+            if (MediaInfoLib::Config.LegacyStreamDisplay_Get())
+            {
+                Infos["Format_Profile"]+=__T(" / LC");
+                Infos["SamplingRate"]+=__T(" / ")+SamplingRate_Previous;
+            }
         }
-        Infos["Format_Settings_SBR"]=__T("Yes (Implicit)");
+        Infos["Format_Settings"]=sbrData?__T("Explicit"):__T("NBC"); // "Not Backward Compatible"
+        Infos["Format_Settings_SBR"]=sbrData?__T("Yes (Explicit)"):__T("Yes (NBC)"); // "Not Backward Compatible"
         Infos["Codec"]=Ztring().From_Local(Aac_audioObjectType(audioObjectType))+__T("-SBR");
     }
     else if (sbrData)
         Infos["Format_Settings_SBR"]=__T("No (Explicit)");
 
-    if (psPresentFlag)
+    if (psPresentFlag || !Infos["Format_Settings_PS"].empty())
     {
         Infos["Format_Profile"]=__T("HE-AACv2");
         Ztring Channels=Infos["Channel(s)"];
         Ztring ChannelPositions=Infos["ChannelPositions"];
-        Ztring SamplingRate=Infos["SamplingRate"];
+        Ztring SamplingRate_Previous=Infos["SamplingRate"];
         Infos["Channel(s)"]=__T("2");
         Infos["ChannelPositions"]=__T("Front: L R");
         if (MediaInfoLib::Config.LegacyStreamDisplay_Get())
@@ -550,9 +564,17 @@ void File_Aac::AudioSpecificConfig_OutOfBand (int32u sampling_frequency_, int8u
             Infos["Format_Profile"]+=__T(" / HE-AAC / LC");
             Infos["Channel(s)"]+=__T(" / ")+Channels+__T(" / ")+Channels;
             Infos["ChannelPositions"]+=__T(" / ")+ChannelPositions+__T(" / ")+ChannelPositions;
-            Infos["SamplingRate"]=Ztring().From_Number((extension_sampling_frequency_index==(int8u)-1)?(sampling_frequency*2):extension_sampling_frequency, 10)+__T(" / ")+SamplingRate;
+            int32u SamplingRate=(extension_sampling_frequency_index==(int8u)-1)?(((int32u)Frequency_b)*2):extension_sampling_frequency;
+            if (SamplingRate)
+                Infos["SamplingRate"]=Ztring().From_Number(SamplingRate, 10)+__T(" / ")+SamplingRate_Previous;
         }
-        Infos["Format_Settings_PS"]=__T("Yes (Implicit)");
+        if (Infos["Format_Settings"]!=(psData?__T("Explicit"):__T("NBC")))
+        {
+            if (!Infos["Format_Settings"].empty())
+                Infos["Format_Settings"].insert(0, __T(" / "));
+            Infos["Format_Settings"].insert(0, psData?__T("Explicit"):__T("NBC")); // "Not Backward Compatible"
+        }
+        Infos["Format_Settings_PS"]=psData?__T("Yes (Explicit)"):__T("Yes (NBC)"); // "Not Backward Compatible"
         if (StreamPos_Last!=(size_t)-1)
         {
             Ztring Codec=Retrieve(Stream_Audio, StreamPos_Last, Audio_Codec);
@@ -594,6 +616,7 @@ void File_Aac::AudioMuxElement()
     }
     if (sampling_frequency_index==(int8u)-1) //No previous config
     {
+        CanFill=false;
         Skip_BS(Data_BS_Remain(),                               "(Waiting for configuration)");
         return;
     }
@@ -1072,7 +1095,7 @@ void File_Aac::adts_fixed_header()
     Get_SB (    protection_absent,                              "protection_absent");
     Get_S1 ( 2, audioObjectType,                                "profile_ObjectType"); audioObjectType++; Param_Info1(Aac_audioObjectType(audioObjectType));
     Get_S1 ( 4, sampling_frequency_index,                       "sampling_frequency_index"); Param_Info2(Aac_sampling_frequency[sampling_frequency_index], " Hz");
-    sampling_frequency=Aac_sampling_frequency[sampling_frequency_index];
+    Frequency_b=Aac_sampling_frequency[sampling_frequency_index];
     Skip_SB(                                                    "private");
     Get_S1 ( 3, channelConfiguration,                           "channel_configuration");
     Skip_SB(                                                    "original");
@@ -1088,10 +1111,12 @@ void File_Aac::adts_fixed_header()
             Infos["Format_Version"].From_Local(id?"Version 2":"Version 4");
             Infos["Format_Profile"].From_Local(Aac_Format_Profile(audioObjectType));
             Infos["Codec"].From_Local(Aac_audioObjectType(audioObjectType));
-            Infos["SamplingRate"].From_Number(Aac_sampling_frequency[sampling_frequency_index]);
+            if (Aac_sampling_frequency[sampling_frequency_index])
+                Infos["SamplingRate"].From_Number(Aac_sampling_frequency[sampling_frequency_index]);
             Infos["Channel(s)"].From_Number(channelConfiguration);
             Infos["ChannelPositions"].From_Local(Aac_ChannelConfiguration[channelConfiguration]);
             Infos["ChannelPositions/String2"].From_Local(Aac_ChannelConfiguration2[channelConfiguration]);
+            Infos["ChannelLayout"].From_Local(Aac_ChannelLayout[channelConfiguration]);
             if (IsSub)
                 Infos["MuxingMode"].From_Local("ADTS");
         }
@@ -1112,25 +1137,11 @@ void File_Aac::adts_variable_header()
     Element_End0();
 
     FILLING_BEGIN();
-        aac_frame_lengths.push_back(aac_frame_length);
-
         if (adts_buffer_fullness==0x7FF)
-            Infos["BitRate_Mode"].From_Local("VBR");
-        else
-        {
-            //Calculating
-            int64u aac_frame_length_Total=0;
-            for (size_t Pos=0; Pos<aac_frame_lengths.size(); Pos++)
-                aac_frame_length_Total+=aac_frame_lengths[Pos];
-
-            int64u BitRate=(Aac_sampling_frequency[sampling_frequency_index]/1024);
-            BitRate*=aac_frame_length_Total*8;
-            BitRate/=aac_frame_lengths.size();
-
-            //Filling
-            Infos["BitRate_Mode"].From_Local("CBR");
-            Infos["BitRate"].From_Number(BitRate);
-        }
+            adts_buffer_fullness_Is7FF=true;
+        #if MEDIAINFO_ADVANCED
+            aac_frame_length_Total+=aac_frame_length;
+        #endif //MEDIAINFO_ADVANCED
     FILLING_END();
 }
 
