@@ -42,7 +42,7 @@
 #endif //defined(MEDIAINFO_PBCORE_YES)
 #include "MediaInfo/MediaInfo_Internal.h"
 #include "MediaInfo/File__Analyze.h"
-#include "base64.h"
+#include "ThirdParty/base64/base64.h"
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -90,10 +90,11 @@ extern MediaInfo_Config Config;
 //---------------------------------------------------------------------------
 Ztring MediaInfo_Internal::Inform()
 {
-    CS.Enter();
+    {
+        CriticalSectionLocker CSL(CS);
     if (Info && Info->Status[File__Analyze::IsUpdated])
         Info->Open_Buffer_Update();
-    CS.Leave();
+    }
 
     #if MEDIAINFO_TRACE
         if (MediaInfoLib::Config.Inform_Get()!=__T("MAXML") && (MediaInfoLib::Config.Trace_Level_Get() || MediaInfoLib::Config.Inform_Get()==__T("Details")))
@@ -112,6 +113,22 @@ Ztring MediaInfo_Internal::Inform()
             return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_5);
         if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.6"))
             return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_6);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_8, (Export_EbuCore::acquisitiondataoutputmode)MediaInfoLib::Config.AcquisitionDataOutputMode_Get());
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_parameterSegment") || MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_ps"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_8, Export_EbuCore::AcquisitionDataOutputMode_parameterSegment);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_segmentParameter") || MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_sp"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_8, Export_EbuCore::AcquisitionDataOutputMode_segmentParameter);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.5_JSON"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_5, Export_EbuCore::AcquisitionDataOutputMode_Default, Export_EbuCore::Format_JSON);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.6_JSON"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_6, Export_EbuCore::AcquisitionDataOutputMode_Default, Export_EbuCore::Format_JSON);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_JSON"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_8, (Export_EbuCore::acquisitiondataoutputmode)MediaInfoLib::Config.AcquisitionDataOutputMode_Get(), Export_EbuCore::Format_JSON);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_parameterSegment_JSON") || MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_ps_JSON"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_8, Export_EbuCore::AcquisitionDataOutputMode_parameterSegment, Export_EbuCore::Format_JSON);
+        if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_segmentParameter_JSON") || MediaInfoLib::Config.Inform_Get()==__T("EBUCore_1.8_sp_JSON"))
+            return Export_EbuCore().Transform(*this, Export_EbuCore::Version_1_8, Export_EbuCore::AcquisitionDataOutputMode_segmentParameter, Export_EbuCore::Format_JSON);
         if (MediaInfoLib::Config.Inform_Get()==__T("EBUCore"))
             return Export_EbuCore().Transform(*this);
     #endif //defined(MEDIAINFO_EBUCORE_YES)
@@ -147,7 +164,7 @@ Ztring MediaInfo_Internal::Inform()
      && MediaInfoLib::Config.Inform_Get(__T("Video")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Audio")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Text")).empty()
-     && MediaInfoLib::Config.Inform_Get(__T("Chapters")).empty()
+     && MediaInfoLib::Config.Inform_Get(__T("Other")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Image")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Menu")).empty()
      ))
@@ -188,15 +205,15 @@ Ztring MediaInfo_Internal::Inform()
         if (Count_Get(Stream_Text))
             Retour+=MediaInfoLib::Config.Inform_Get(__T("Text_End"));
         if (Count_Get(Stream_Other))
-            Retour+=MediaInfoLib::Config.Inform_Get(__T("Chapters_Begin"));
+            Retour+=MediaInfoLib::Config.Inform_Get(__T("Other_Begin"));
         for (size_t I1=0; I1<Count_Get(Stream_Other); I1++)
         {
             Retour+=Inform(Stream_Other, I1, false);
             if (I1!=Count_Get(Stream_Other)-1)
-                Retour+=MediaInfoLib::Config.Inform_Get(__T("Chapters_Middle"));
+                Retour+=MediaInfoLib::Config.Inform_Get(__T("Other_Middle"));
         }
         if (Count_Get(Stream_Other))
-            Retour+=MediaInfoLib::Config.Inform_Get(__T("Chapters_End"));
+            Retour+=MediaInfoLib::Config.Inform_Get(__T("Other_End"));
         if (Count_Get(Stream_Image))
             Retour+=MediaInfoLib::Config.Inform_Get(__T("Image_Begin"));
         for (size_t I1=0; I1<Count_Get(Stream_Image); I1++)
@@ -387,7 +404,7 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
      && MediaInfoLib::Config.Inform_Get(__T("Video")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Audio")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Text")).empty()
-     && MediaInfoLib::Config.Inform_Get(__T("Chapters")).empty()
+     && MediaInfoLib::Config.Inform_Get(__T("Other")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Image")).empty()
      && MediaInfoLib::Config.Inform_Get(__T("Menu")).empty())
     {
@@ -430,13 +447,16 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
             #if defined(MEDIAINFO_XML_YES)
             if (XML_0_7_78)
             {
+                Ztring Options=Get((stream_t)StreamKind, StreamPos, Champ_Pos, Info_Options);
                 if (Champ_Pos>=Stream[StreamKind][StreamPos].size())
-                    Shouldshow=true;
+                {
+                    if (InfoOption_ShowInXml>=Options.size() || Options[InfoOption_ShowInXml]==__T('Y'))
+                        Shouldshow=true;
+                }
                 else
                 {
-                Ztring Options=Get((stream_t)StreamKind, StreamPos, Champ_Pos, Info_Options);
-                if (InfoOption_ShowInXml<Options.size() && Options[InfoOption_ShowInXml]==__T('Y'))
-                    Shouldshow=true;
+                    if (InfoOption_ShowInXml<Options.size() && Options[InfoOption_ShowInXml]==__T('Y'))
+                        Shouldshow=true;
                 }
             }
             else
@@ -526,6 +546,16 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
                         if (SlashPos!=string::npos)
                             Valeur.erase(SlashPos);
                     }
+                    Ztring Format_Profile_More;
+                    if (XML_0_7_78 && Nom==__T("Format_Profile"))
+                    {
+                        size_t SeparatorPos=Valeur.find(__T('@'));
+                        if (SeparatorPos!=string::npos && Valeur.find(__T(" / "))==string::npos) //TODO: better support of compatibility modes (e.g. "Multiview") and sequences (e.g. different profiles in different files "BCS@L3 / BCS@L2 / BCS@L3")
+                        {
+                            Format_Profile_More=Valeur.substr(SeparatorPos+1);
+                            Valeur.erase(SeparatorPos);
+                        }
+                    }
 
                     Retour+=__T("<");
                     Retour+=Nom;
@@ -539,6 +569,27 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
                     Retour+=__T("</");
                     Retour+=Nom;
                     Retour+=__T(">");
+
+                    if (!Format_Profile_More.empty())
+                    {
+                        if (Format_Profile_More.size()>=2 && Format_Profile_More[0]==__T('L') && Format_Profile_More[1]>=__T('0') && Format_Profile_More[1]<=__T('9'))
+                            Format_Profile_More.erase(0, 1);
+                        size_t SeparatorPos=Format_Profile_More.find(__T('@'));
+                        if (SeparatorPos!=string::npos)
+                        {
+                            Retour+=__T("\n<Format_Level>");
+                            Retour+=Format_Profile_More.substr(0, SeparatorPos);
+                            Retour+=__T("</Format_Level>\n<Format_Tier>");
+                            Retour+=Format_Profile_More.substr(SeparatorPos+1);
+                            Retour+=__T("</Format_Tier>");
+                        }
+                        else
+                        {
+                            Retour+=__T("\n<Format_Level>");
+                            Retour+=Format_Profile_More;
+                            Retour+=__T("</Format_Level>");
+                        }
+                    }
                 }
                 #endif //defined(MEDIAINFO_XML_YES)
                 #if defined(MEDIAINFO_CSV_YES)
@@ -589,9 +640,9 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
 
     //Gestion $xx$
     size_t PosX=0;
-    while (Retour.find(__T("$"), PosX)!=(size_t)-1)
+    while (Retour.find(__T('$'), PosX)!=(size_t)-1)
     {
-        PosX=Retour.find(__T("$"), PosX);
+        PosX=Retour.find(__T('$'), PosX);
         if (Retour.size()>PosX+2 && !(Retour(PosX+1)==__T('i') && Retour(PosX+2)==__T('f') && Retour(PosX+3)==__T('('))) //To keep out "%" without any signification, or "$if(..."
         {
             Ztring ARemplacer=Ztring(__T("$")+Retour.SubString(__T("$"), __T("$"), PosX))+__T("$");
@@ -668,10 +719,12 @@ Ztring MediaInfo_Internal::Inform (stream_t StreamKind, size_t StreamPos, bool I
 
     //Gestion %xxx%
     PosX=0;
-    while (Retour.find(__T("%"), PosX)!=(size_t)-1)
+    while (Retour.find(__T('%'), PosX)!=(size_t)-1)
     {
-        PosX=Retour.find(__T("%"), PosX);
-        if (Retour.size() > PosX + 2 && Retour[PosX + 1] >= __T('A') && Retour[PosX + 1] <= __T('Z')) //To keep out "%" without any signification
+        PosX=Retour.find(__T('%'), PosX);
+        if (Retour.size() > PosX + 2
+         && ((Retour[PosX + 1] >= __T('a') && Retour[PosX + 1] <= __T('z'))
+          || (Retour[PosX + 1] >= __T('A') && Retour[PosX + 1] <= __T('Z')))) //To keep out "%" without any signification
         {
             Ztring ARemplacer = Ztring(__T("%") + Retour.SubString(__T("%"), __T("%"), PosX)) + __T("%");
             Ztring RemplacerPar = Get(StreamKind, StreamPos, Retour.SubString(__T("%"), __T("%"), PosX));
