@@ -199,7 +199,6 @@ private :
     void moov_trak_mdia_minf_stbl_stss();
     void moov_trak_mdia_minf_stbl_stsz();
     void moov_trak_mdia_minf_stbl_stts();
-    void moov_trak_mdia_minf_stbl_stts_Common(int32u SampleCount, int32u SampleDuration, int32u Pos=0, int32u NumberOfEntries=1);
     void moov_trak_mdia_minf_stbl_stz2() {moov_trak_mdia_minf_stbl_stsz();}
     void moov_trak_meta() {moov_meta();}
     void moov_trak_meta_hdlr() {moov_meta_hdlr();}
@@ -354,6 +353,7 @@ private :
         {
             int32u TimeScale;
             int32u FrameDuration;
+            int8u  NumberOfFrames;
             bool   DropFrame;
             bool   H24;
             bool   NegativeTimes;
@@ -444,10 +444,12 @@ private :
             size_t          stts_Durations_Pos;
             int64u          stts_FramePos;
         #endif //MEDIAINFO_DEMUX || MEDIAINFO_SEEK
+        void                moov_trak_mdia_minf_stbl_stts_Common(int32u SampleCount, int32u SampleDuration, int32u Pos=0, int32u NumberOfEntries=1);
         #if MEDIAINFO_DEMUX
             bool            PtsDtsAreSame;
             bool            Demux_EventWasSent;
             int32u          CodecID;
+            void            SplitAudio(File_Mpeg4::stream& Video, int32u moov_mvhd_TimeScale);
         #endif //MEDIAINFO_DEMUX
 
         stream()
@@ -482,6 +484,7 @@ private :
             IsPriorityStream=false;
             IsFilled=false;
             IsChapter=false;
+            IsEnabled=false;
             IsExcluded=false;
             HasForcedSamples=false;
             AllForcedSamples=false;
@@ -530,11 +533,20 @@ private :
         int32u StreamID;
         int32u Reserved1;
         int64u Reserved2;
+        bool operator<(const mdat_Pos_Type& r) const
+        {
+            return Offset<r.Offset;
+        }
     };
     typedef std::vector<mdat_Pos_Type> mdat_pos;
     static bool mdat_pos_sort (const File_Mpeg4::mdat_Pos_Type &i,const File_Mpeg4::mdat_Pos_Type &j) { return (i.Offset<j.Offset); }
+    void IsParsing_mdat_Set();
+    #if MEDIAINFO_DEMUX
+    void TimeCodeTrack_Check(stream &Stream_Temp, size_t Pos, int32u StreamID);
+    #endif //MEDIAINFO_DEMUX
     mdat_pos mdat_Pos;
     mdat_Pos_Type* mdat_Pos_Temp;
+    mdat_Pos_Type* mdat_Pos_Temp_ToJump;
     mdat_Pos_Type* mdat_Pos_Max;
     std::vector<int32u> mdat_Pos_ToParseInPriority_StreamIDs;
     bool                mdat_Pos_NormalParsing;
