@@ -408,6 +408,7 @@ private :
     void sei_message_user_data_unregistered_bluray(int32u payloadSize);
     void sei_message_recovery_point();
     void sei_message_mainconcept(int32u payloadSize);
+    void sei_alternative_transfer_characteristics();
     void access_unit_delimiter();
     void filler_data();
     void prefix_nal_unit(bool svc_extension_flag);
@@ -451,22 +452,6 @@ private :
     //Temporal references
     struct temporal_reference
     {
-        struct buffer_data
-        {
-            size_t Size;
-            int8u* Data;
-
-            buffer_data()
-            {
-                Size=0;
-                Data=NULL;
-            }
-
-            ~buffer_data()
-            {
-                delete[] Data; //Data=NULL;
-            }
-        };
         #if defined(MEDIAINFO_DTVCCTRANSPORT_YES)
             buffer_data* GA94_03;
         #endif //MEDIAINFO_DTVCCTRANSPORT_YES
@@ -550,7 +535,9 @@ private :
     int32u                              Firstpic_order_cnt_lsbInBlock;
     int8u                               nal_ref_idc;
     int8u                               FrameRate_Divider;
+    int8u                               preferred_transfer_characteristics;
     bool                                FirstPFrameInGop_IsParsed;
+    bool                                Config_IsRepeated;
     #if MEDIAINFO_ADVANCED2
         std::vector<std::string>        Dump_SPS;
         std::vector<std::string>        Dump_PPS;
